import RPi.GPIO as GPIO
from datetime import datetime
import time
import requests
import os
from multiprocessing import Process, Manager

GPIO.setwarnings(False)
GPIO.setmode(GPIO.BOARD)
unlocked = True

class keypad():
    def __init__(self, columnCount = 3):

        # CONSTANTS 
        if columnCount == 3:
            self.KEYPAD = [
                [1,2,3],
                [4,5,6],
                [7,8,9],
                ["*",0,"#"]
            ]

            self.ROW         = [26,24,23,22]
            self.COLUMN      = [21,19,10]
        
        elif columnCount == 4:
            self.KEYPAD = [
                [1,2,3,"A"],
                [4,5,6,"B"],
                [7,8,9,"C"],
                ["*",0,"#","D"]
            ]

            self.ROW         = [3,5,7,29]
            self.COLUMN      = [31,26,24,21]
        else:
            return
     
    def getKey(self):
         
        # Set all columns as output low
        for j in range(len(self.COLUMN)):
            GPIO.setup(self.COLUMN[j], GPIO.OUT)
            GPIO.output(self.COLUMN[j], GPIO.LOW)
         
        # Set all rows as input
        for i in range(len(self.ROW)):
            GPIO.setup(self.ROW[i], GPIO.IN, pull_up_down=GPIO.PUD_UP)
         
        # Scan rows for pushed key/button
        # A valid key press should set "rowVal"  between 0 and 3.
        rowVal = -1
        for i in range(len(self.ROW)):
            tmpRead = GPIO.input(self.ROW[i])
            if tmpRead == 0:
                rowVal = i
                 
        # if rowVal is not 0 thru 3 then no button was pressed and we can exit
        if rowVal <0 or rowVal >3:
            self.exit()
            return
         
        # Convert columns to input
        for j in range(len(self.COLUMN)):
                GPIO.setup(self.COLUMN[j], GPIO.IN, pull_up_down=GPIO.PUD_DOWN)
         
        # Switch the i-th row found from scan to output
        GPIO.setup(self.ROW[rowVal], GPIO.OUT)
        GPIO.output(self.ROW[rowVal], GPIO.HIGH)
 
        # Scan columns for still-pushed key/button
        # A valid key press should set "colVal"  between 0 and 2.
        colVal = -1
        for j in range(len(self.COLUMN)):
            tmpRead = GPIO.input(self.COLUMN[j])
            if tmpRead == 1:
                colVal = j
                 
        # if colVal is not 0 thru 2 then no button was pressed and we can exit
        if colVal < 0 or colVal > 3:
            self.exit()
            return
 
        # Return the value of the key pressed
        self.exit()
        return self.KEYPAD[rowVal][colVal]
         
    def exit(self):
        # Reinitialize all rows and columns as input at exit
        for i in range(len(self.ROW)):
                GPIO.setup(self.ROW[i], GPIO.IN, pull_up_down=GPIO.PUD_UP) 
        for j in range(len(self.COLUMN)):
                GPIO.setup(self.COLUMN[j], GPIO.IN, pull_up_down=GPIO.PUD_UP)
         
def wrong_code_ifttt_notification():
    requests.post("https://maker.ifttt.com/trigger/wrong_code/with/key/lwCBZq7PMkYtevPPNGQ9zOO6gKI1jqps61dvGGFIXX7")

def box_opened_ifttt_notification():
    requests.post("https://maker.ifttt.com/trigger/box_opened/with/key/lwCBZq7PMkYtevPPNGQ9zOO6gKI1jqps61dvGGFIXX7")

def take_a_picture():
    now = datetime.now()
    current_time = now.strftime("%Y%m%d%H%M%S")
    filepath = '/home/pi/pictures/Security_' + current_time + '.jpg'
    command = 'fswebcam -r 1920x1080 --no-banner ' + filepath
    os.system(command)
    print('Its taking a picture')

def verify_password(kp):
    ###### 4 Digit wait ######
    seq = []
    while len(seq) != 4 or len(seq) > 4:
        digit = None
        while digit == None:
            digit = kp.getKey()
            if digit != None and isinstance(digit, int):
                seq.append(digit)
            time.sleep(0.5)
 
    # Check digit code
    l_unlocked = False
    if seq == [1, 2, 3, 4]:
        print("Code accepted")
        l_unlocked = True
    else:
        print("Wrong Code")
        take_a_picture()
        wrong_code_ifttt_notification()
	
    return l_unlocked

def run_box(p_unlocked):
    kp = keypad(columnCount=4)
    while True:
        digit = None
        run_key = "A"
        lock_key = "B"
        l_unlocked = p_unlocked.value
        while digit == None:
            digit = kp.getKey()
            p_unlocked.value = l_unlocked
            time.sleep(0.2)
        
        if l_unlocked == False and digit == run_key:
            print('Please put in your password')
            l_unlocked = verify_password(kp)
    
        if digit == lock_key:
            print('The box is locked!')
            l_unlocked = False

        p_unlocked.value = l_unlocked

def get_distance(p_distance):
    while True:
        TRIG = 23
        ECHO = 19
        GPIO.setup(TRIG,GPIO.OUT)
        GPIO.setup(ECHO,GPIO.IN)
        
        GPIO.output(TRIG, False)
    
        time.sleep(2)
        GPIO.output(TRIG, True)
        time.sleep(0.00001)
        GPIO.output(TRIG, False)
        
        while GPIO.input(ECHO) == 0:
            pulse_start = time.time()
        
        while GPIO.input(ECHO) == 1:
            pulse_end = time.time()
        
        pulse_duration = pulse_end - pulse_start
        l_distance = pulse_duration * 17165
        l_distance = round(l_distance, 1)
        #print('Distance: ', l_distance, 'cm')
        p_distance.value = l_distance

if __name__ == '__main__':
    #kp = keypad(columnCount=4)
    running = True
    manager = Manager()
    unlocked = manager.Value('unlocked', unlocked)
    distance = manager.Value('distance', 0)
    p_run_box = Process(target=run_box, args=(unlocked,))
    p_get_distance = Process(target=get_distance,  args=(distance,))
    p_get_distance.start()
    p_run_box.start()
    currently_unlocked = not unlocked.value
    while running:
        time.sleep(2)
        if distance.value > 10 and unlocked.value == False and currently_unlocked == unlocked.value:
            box_opened_ifttt_notification()
            take_a_picture()
            currently_unlocked != unlocked.value
